import 'package:flutter/widgets.dart';
import 'package:provider/provider.dart';
import 'package:bloc_stream/bloc_stream.dart';

class BlocStreamProvider<T extends BlocStream<dynamic>>
    extends ValueDelegateWidget<T> implements SingleChildCloneableWidget {
  final Widget child;

  BlocStreamProvider({
    Key key,
    @required ValueBuilder<T> create,
    Widget child,
  }) : this._(
          key: key,
          delegate: BuilderStateDelegate<T>(
            create,
            dispose: (_, stream) => stream?.close(),
          ),
          child: child,
        );

  BlocStreamProvider.value({
    Key key,
    @required T value,
    Widget child,
  }) : this._(
          key: key,
          delegate: SingleValueDelegate<T>(value),
          child: child,
        );

  BlocStreamProvider._({
    Key key,
    @required ValueStateDelegate<T> delegate,
    this.child,
  }) : super(key: key, delegate: delegate);

  static T of<T extends BlocStream<dynamic>>(BuildContext context) {
    try {
      return Provider.of<T>(context, listen: false);
    } on Object catch (_) {
      throw FlutterError(
        '''
        BlocStreamProvider.of() called with a context that does not contain a BlocStream of type $T.
        No ancestor could be found starting from the context that was passed to BlocStreamProvider.of<$T>().

        The context used was: $context
        ''',
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    return InheritedProvider<T>(
      value: delegate.value,
      child: child,
    );
  }

  @override
  BlocStreamProvider<T> cloneWithChild(Widget child) {
    return BlocStreamProvider<T>._(
      key: key,
      delegate: delegate,
      child: child,
    );
  }
}
